"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkGenerator = exports.NetworkSettingsConfig = void 0;
const logger_1 = require("../../../lib/logger");
const network_signals_filter_1 = require("./network-signals-filter");
const types_1 = require("../../../types");
const network_interceptor_1 = require("./network-interceptor");
const helpers_1 = require("./helpers");
class NetworkSettingsConfig {
    constructor({ networkSignalsAllowList, networkSignalsDisallowList, networkSignalsAllowSameDomain, }) {
        this.networkSignalsAllowSameDomain = networkSignalsAllowSameDomain ?? true;
        this.networkSignalsFilterList = new network_signals_filter_1.NetworkSignalsFilterList(networkSignalsAllowList, networkSignalsDisallowList);
    }
}
exports.NetworkSettingsConfig = NetworkSettingsConfig;
class NetworkGenerator {
    constructor(settings) {
        this.id = 'network';
        this.interceptor = new network_interceptor_1.NetworkInterceptor();
        this.filter = new network_signals_filter_1.NetworkSignalsFilter(settings);
    }
    register(emitter) {
        const createMetadata = () => ({
            filters: this.filter.settings.networkSignalsFilterList.getRegexes(),
        });
        const handleFetchRequest = ([url, rq]) => {
            if (!rq || !rq.body) {
                return;
            }
            if (!(0, helpers_1.containsJSONContentType)(rq.headers)) {
                return;
            }
            const sUrl = url?.toString();
            if (!url || !this.filter.isAllowed(sUrl)) {
                return;
            }
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'request',
                url: sUrl,
                method: rq.method || 'GET',
                data: JSON.parse(rq.body.toString()),
            }, createMetadata()));
        };
        const handleFetchResponse = async (rs) => {
            if (rs.status < 200 || rs.status >= 300) {
                return;
            }
            if (!(0, helpers_1.containsJSONContentType)(rs.headers)) {
                return;
            }
            const url = rs.url;
            if (!url || !this.filter.isAllowed(url)) {
                return;
            }
            const data = await rs.json();
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'response',
                url,
                data: data,
            }, createMetadata()));
        };
        this.interceptor.addFetchInterceptor(handleFetchRequest, handleFetchResponse);
        const handleXHRRequest = ([url, rq]) => {
            if (!rq || !rq.body) {
                return;
            }
            if (!(0, helpers_1.containsJSONContentType)(rq.headers)) {
                return;
            }
            const sUrl = url?.toString();
            if (!url || !this.filter.isAllowed(sUrl)) {
                return;
            }
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'request',
                url: sUrl,
                method: rq.method,
                data: tryParseXHRBody(rq.body),
            }, createMetadata()));
        };
        const handleXHRResponse = ({ body, headers, status, url, responseType, }) => {
            if (status < 200 || status >= 300) {
                return;
            }
            if (responseType !== 'json' && !(0, helpers_1.containsJSONContentType)(headers)) {
                return;
            }
            if (!url || !this.filter.isAllowed(url)) {
                return;
            }
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'response',
                url: url,
                data: tryParseXHRBody(body),
            }, createMetadata()));
        };
        this.interceptor.addXhrInterceptor(handleXHRRequest, handleXHRResponse);
        return () => {
            this.interceptor.cleanup();
            logger_1.logger.debug('Removing fetch interceptor');
        };
    }
}
exports.NetworkGenerator = NetworkGenerator;
function isPlainObject(obj) {
    return (Object.prototype.toString.call(obj).slice(8, -1).toLowerCase() === 'object');
}
const tryParseXHRBody = (body) => {
    if (!body) {
        return null;
    }
    if (isPlainObject(body) || Array.isArray(body)) {
        return body;
    }
    try {
        return JSON.parse(body.toString());
    }
    catch (e) {
        return null;
    }
};
//# sourceMappingURL=index.js.map