import { logger } from '../../../lib/logger';
import { NetworkSignalsFilter, NetworkSignalsFilterList, } from './network-signals-filter';
import { createNetworkSignal } from '../../../types';
import { NetworkInterceptor, } from './network-interceptor';
import { containsJSONContentType } from './helpers';
export class NetworkSettingsConfig {
    constructor({ networkSignalsAllowList, networkSignalsDisallowList, networkSignalsAllowSameDomain, }) {
        this.networkSignalsAllowSameDomain = networkSignalsAllowSameDomain ?? true;
        this.networkSignalsFilterList = new NetworkSignalsFilterList(networkSignalsAllowList, networkSignalsDisallowList);
    }
}
export class NetworkGenerator {
    constructor(settings) {
        this.id = 'network';
        this.interceptor = new NetworkInterceptor();
        this.filter = new NetworkSignalsFilter(settings);
    }
    register(emitter) {
        const createMetadata = () => ({
            filters: this.filter.settings.networkSignalsFilterList.getRegexes(),
        });
        const handleFetchRequest = ([url, rq]) => {
            if (!rq || !rq.body) {
                return;
            }
            if (!containsJSONContentType(rq.headers)) {
                return;
            }
            const sUrl = url?.toString();
            if (!url || !this.filter.isAllowed(sUrl)) {
                return;
            }
            emitter.emit(createNetworkSignal({
                action: 'request',
                url: sUrl,
                method: rq.method || 'GET',
                data: JSON.parse(rq.body.toString()),
            }, createMetadata()));
        };
        const handleFetchResponse = async (rs) => {
            if (rs.status < 200 || rs.status >= 300) {
                return;
            }
            if (!containsJSONContentType(rs.headers)) {
                return;
            }
            const url = rs.url;
            if (!url || !this.filter.isAllowed(url)) {
                return;
            }
            const data = await rs.json();
            emitter.emit(createNetworkSignal({
                action: 'response',
                url,
                data: data,
            }, createMetadata()));
        };
        this.interceptor.addFetchInterceptor(handleFetchRequest, handleFetchResponse);
        const handleXHRRequest = ([url, rq]) => {
            if (!rq || !rq.body) {
                return;
            }
            if (!containsJSONContentType(rq.headers)) {
                return;
            }
            const sUrl = url?.toString();
            if (!url || !this.filter.isAllowed(sUrl)) {
                return;
            }
            emitter.emit(createNetworkSignal({
                action: 'request',
                url: sUrl,
                method: rq.method,
                data: tryParseXHRBody(rq.body),
            }, createMetadata()));
        };
        const handleXHRResponse = ({ body, headers, status, url, responseType, }) => {
            if (status < 200 || status >= 300) {
                return;
            }
            if (responseType !== 'json' && !containsJSONContentType(headers)) {
                return;
            }
            if (!url || !this.filter.isAllowed(url)) {
                return;
            }
            emitter.emit(createNetworkSignal({
                action: 'response',
                url: url,
                data: tryParseXHRBody(body),
            }, createMetadata()));
        };
        this.interceptor.addXhrInterceptor(handleXHRRequest, handleXHRResponse);
        return () => {
            this.interceptor.cleanup();
            logger.debug('Removing fetch interceptor');
        };
    }
}
function isPlainObject(obj) {
    return (Object.prototype.toString.call(obj).slice(8, -1).toLowerCase() === 'object');
}
const tryParseXHRBody = (body) => {
    if (!body) {
        return null;
    }
    if (isPlainObject(body) || Array.isArray(body)) {
        return body;
    }
    try {
        return JSON.parse(body.toString());
    }
    catch (e) {
        return null;
    }
};
//# sourceMappingURL=index.js.map